/***********************************************************************
 * Time-stamp: <Wednesday, 18.07.2001 at 19:27:21; edited by bauke>
 * 
 * Tina's random number generators
 *
 * lagged Fibonacci generator
 *
 * Copyright by Heiko Bauke, Magdeburg 2001 
 *
 * heiko.bauke@student.uni-magdeburg.de
 *
 * This is free software. You may use and distribute this software under
 * the terms of the GNU GENERAL PUBLIC LICENSE Version 2. See COPYING
 * for details.
 *
 ***********************************************************************/

#ifndef FIBONACCI_H
#define FIBONACCI_H

#include <iostream>
#include <strstream>
#include <new>
#include "trng.h"

class Fibonacci : public TRNG::RNG {
private:
  std::vector<long> r;
  long p, q, pointer, steps;
  TRNG::ParkMiller R_init;
public:
  void backward(void);
  static const TRNG::RNG_type type=TRNG::user1_t;
  const char * name(void);
  void reset(void);
  void seed(long);
  long rand(void);
  RNG *copy(void);
  void split(long, long);
  void jump(long);
  void save_status(std::vector<long> &);
  void load_status(std::vector<long> &);
  Fibonacci(long, long);
  virtual ~Fibonacci() {};
};

const char * Fibonacci::name(void) {
  std::ostrstream out;
  out << "Fibonacci " << p << " " << q;
  return((const char *)out.str());
}

void Fibonacci::reset(void) {
  steps=1l;
  seed(0l);
}

void Fibonacci::seed(long s) {
  long h1, h2;
  h1=0x7fffffffl;
  h2=0x40000000l;
  R_init.seed(s);
  for (long i=0l; i<p; ++i) {
    if (i<q) {
      r[i]=((R_init.rand()+1l) & h1) | h2;
      h1>>=1;
      h2>>=1;
      if (h2==0) {
	h1=0x7fffffffl;
	h2=0x40000000l;
      }
    } else
      r[i]=R_init.rand()+1l;    
/*     r[i]=0l; */
/*     for (long j=0l; j<31l; ++j) { */
/*       r[i]<<=1l; */
/*       if (R_init.boolean()) */
/*         ++r[i]; */
/*     } */
  }
  pointer=p-1l;
}

void Fibonacci::backward(void) {
  for (long i=0l; i<steps; ++i) {
    r[pointer]=r[pointer]^r[(pointer+q)%p];
    pointer=(pointer-1l+p)%p;
  }
}

long Fibonacci::rand(void) {
  for (long i=0l; i<steps; ++i) {
    pointer=(pointer+1l)%p;
    r[pointer]=r[pointer]^r[(pointer+q)%p];
  }
  return(r[pointer]);
}

TRNG::RNG *Fibonacci::copy(void) {
  std::vector<long> s;
  TRNG::RNG *R=static_cast<RNG *>(new Fibonacci(p, q));
  save_status(s);
  R->load_status(s);
  return(R);
}

void Fibonacci::split(long s, long n) {
  if (s<1l || n>s || n<0l)
    TRNG::error("invalid arguments for Fibonacci::split");
  if (s>1l) {
    for (long i=0l; i<=n; ++i)
      rand();
    steps*=s;
    backward();
  }
}

void Fibonacci::jump(long s) {
  unsigned long long to=1ull<<s;
  for (unsigned long long i=0ull; i<to; ++i) {
    rand();
  }
}

void Fibonacci::save_status(std::vector<long> &s) {
  s.resize(p+5);
  s[0]=type;
  s[1]=q;
  s[2]=p;
  s[3]=pointer;
  s[4]=steps;
  for (int i=0; i<p; ++i)
    s[i+5]=r[i];
}

void Fibonacci::load_status(std::vector<long> &s) {
  long i;
  if (s[0]!=type)
    throw TRNG::error("Fibonacci::load_status wrong parameter");
  else {
    q=s[1];
    p=s[2];
    pointer=s[3];
    steps=s[4];
    r.resize(p);
    for (i=0; i<p; ++i)
      r[i]=s[i+5];
  }
}

Fibonacci::Fibonacci(long p1, long q1) : r() {
  if (p1<0l || q1<0l || p1==q1)
    throw TRNG::error("bad arguments for Fibonacci::Fibonacci");
  if (p1>q1) {
    p=p1;
    q=q1;
  } else {
    p=q1;
    q=p1;
  }
  r.resize(p);
  max_val=0x7fffffffl;
  max_val2=max_val/2l;
  reset();
}

#endif
