/***********************************************************************
 * Time-stamp: <Mittwoch, 09.05.2001, 14:31:36; edited by heiko>
 * 
 * Tina's random number generators
 *
 * profiling Tina's random number generators
 *
 * Copyright by Heiko Bauke, Magdeburg 2001 
 *
 * heiko.bauke@student.uni-magdeburg.de
 *
 * This is free software. You may use and distribute this software under
 * the terms of the GNU GENERAL PUBLIC LICENSE Version 2. See COPYING
 * for details.
 *
 ***********************************************************************/

#include <cstdlib>
#include <iostream>
#include <iomanip>
#include <vector>
#include <ctime>
#include <cmath>
#include "trng.h"

using namespace TRNG;
using namespace std;


void profl(RNG &r, long j) {
  clock_t my_clock;
  long i;
  cout.setf(ios::fixed);
  my_clock=clock();
  for (i=0l; i<j; ++i)
    r.rand();
  my_clock=clock()-my_clock;
#ifdef LaTeX
  cout << " & ";
#else
  cout << '\t';
#endif
  cout << setfill(' ') << setprecision(2) 
       << static_cast<double>(my_clock)/static_cast<double>(CLOCKS_PER_SEC);
  my_clock=clock();
}


void profd(RNG &r, long j) {
  clock_t my_clock;
  long i;
  cout.setf(ios::fixed);
  my_clock=clock();
  for (i=0l; i<j; ++i)
    r.uniform();
  my_clock=clock()-my_clock;
#ifdef LaTeX
  cout << " & ";
#else
  cout << '\t';
#endif
  cout << setfill(' ') << setprecision(2) 
       << static_cast<double>(my_clock)/static_cast<double>(CLOCKS_PER_SEC);
}


void prof_main(RNG &r,long from, long to) {
  long i;
  cout << r.name();
#ifdef LaTeX
  cout << " & long";
#else
  cout << "\tlong";
#endif
  for (i=(1l<<from); i<=(1l<<to); i*=4l)
    profl(r, i);
#ifdef LaTeX
  cout << "\\\\" << endl;
  cout << " & double ";
#else
  cout << endl << "\tdouble ";
#endif
  for (i=(1l<<from); i<=(1l<<to); i*=4l)
    profd(r, i);
#ifdef LaTeX
  cout << "\\\\" << endl;
#else
  cout << endl;
#endif
}


int main(void) {
  const long from=16;
  const long to=24;
  long i;
#ifdef LaTeX
  cout << " & ";
#else
  cout << "\t";
#endif
  for (i=from; i<=to; i+=2l)
#ifdef LaTeX
    cout << " & $2^{" << i << "}$";
#else
    cout << "\t2^" << i;
#endif
#ifdef LaTeX
  cout << "\\\\" << endl;
#else
  cout << endl;
#endif
  ParkMiller r1;
  RAND r2;
  LCG64 r3;
  LMC2 r4;
  LMC3 r5;
  LMC4 r6;
  CLCG2 r7;
  CLCG3 r8;
  CLCG4 r9;
  EINV r10;
  EINVLCG64 r11;
  prof_main(r1, from, to);
  prof_main(r2, from, to);
  prof_main(r3, from, to);
  prof_main(r4, from, to);
  prof_main(r5, from, to);
  prof_main(r6, from, to);
  prof_main(r7, from, to);
  prof_main(r8, from, to);
  prof_main(r9, from, to);
  prof_main(r10, from, to);
  prof_main(r11, from, to);
  return(EXIT_SUCCESS);
}



